@@ -149,7 +149,7 @@ def _process(data_dir: Path, application_id: str, configuration) -> bool:
                 incoming_task, upload_result.to_ffi()
             )
         elif tag == UploadTaskTag.WAIT:
-            time.sleep(incoming_task.wait._0 / 1000)
+            time.sleep(incoming_task.wait / 1000)
         elif tag == UploadTaskTag.DONE:
             return True
 
