@@ -10,6 +10,13 @@ pub struct CoreMetrics {
     pub first_run_date: DatetimeMetric,
     pub first_run_hour: DatetimeMetric,
     pub os: StringMetric,
+
+    /// The number of times we encountered an IO error
+    /// when writing a pending ping to disk.
+    ///
+    /// **Note**: Not a _core_ metric, but an error metric,
+    /// placed here for the lack of a more suitable part in the Glean struct.
+    pub io_errors: CounterMetric,
 }
 
 impl CoreMetrics {
@@ -56,6 +63,15 @@ impl CoreMetrics {
                 disabled: false,
                 dynamic_label: None,
             }),
+
+            io_errors: CounterMetric::new(CommonMetricData {
+                name: "io".into(),
+                category: "glean.error".into(),
+                send_in_pings: vec!["metrics".into()],
+                lifetime: Lifetime::Ping,
+                disabled: false,
+                dynamic_label: None,
+            }),
         }
     }
 }
